@@ -9,10 +9,12 @@
  */
 #ifndef CoinFactorization_H
 #define CoinFactorization_H
+//#define COIN_ONE_ETA_COPY 100
 
 #include <iostream>
 #include <string>
 #include <cassert>
+#include <cstdio>
 #include "CoinFinite.hpp"
 #include "CoinIndexedVector.hpp"
 class CoinPackedMatrix;
@@ -41,7 +43,6 @@ class CoinPackedMatrix;
  */
 
 
-
 class CoinFactorization {
    friend void CoinFactorizationUnitTest( const std::string & mpsDir );
 
@@ -112,7 +113,7 @@ public:
 		       CoinBigIndex estimateNumberElements,
 		       int * indicesRow[],
 		       int * indicesColumn[],
-		       double * elements[],
+		       CoinFactorizationDouble * elements[],
 		  double areaFactor = 0.0);
   /** This is part two of factorization
       Arrays belong to factorization and were returned by part 1
@@ -151,15 +152,17 @@ public:
     return pivotColumn_.array();
   }
   /// Returns address of pivot region
-  inline double *pivotRegion (  ) const {
+  inline CoinFactorizationDouble *pivotRegion (  ) const {
     return pivotRegion_.array();
   }
   /// Returns address of permuteBack region
   inline int *permuteBack (  ) const {
     return permuteBack_.array();
   }
-  /// Returns address of pivotColumnBack region (also used for permuting)
+  /** Returns address of pivotColumnBack region (also used for permuting)
+      Now uses firstCount to save memory allocation */
   inline int *pivotColumnBack (  ) const {
+    //return firstCount_.array();
     return pivotColumnBack_.array();
   }
   /// Start of each row in L
@@ -179,7 +182,7 @@ public:
   { return indexRowL_.array();}
 
   /// Elements in L (row copy)
-  inline double * elementByRowL() const
+  inline CoinFactorizationDouble * elementByRowL() const
   { return elementByRowL_.array();}
 
   /// Number of Rows after iterating
@@ -324,7 +327,7 @@ public:
   inline int * numberInColumn() const
   { return numberInColumn_.array();}
   /// Elements of U
-  inline double * elementU() const
+  inline CoinFactorizationDouble * elementU() const
   { return elementU_.array();}
   /// Row indices of U
   inline int * indexRowU() const
@@ -366,6 +369,13 @@ public:
 		      int pivotRow,
 		      double pivotCheck ,
 		      bool checkBeforeModifying=false);
+  /** Combines BtranU and delete elements
+      If deleted is NULL then delete elements
+      otherwise store where elements are
+  */
+  void replaceColumnU ( CoinIndexedVector * regionSparse,
+			CoinBigIndex * deleted,
+			int internalPivotRow);
   //@}
 
   /**@name various uses of factorization (return code number elements) 
@@ -484,11 +494,17 @@ public:
   /// Does most of factorization
   int factor (  );
 protected:
-  /// Does sparse phase of factorization
-  /// return code is <0 error, 0= finished
+  /** Does sparse phase of factorization
+      return code is <0 error, 0= finished */
   int factorSparse (  );
-  /// Does dense phase of factorization
-  /// return code is <0 error, 0= finished
+  /** Does sparse phase of factorization (for smaller problems)
+      return code is <0 error, 0= finished */
+  int factorSparseSmall (  );
+  /** Does sparse phase of factorization (for larger problems)
+      return code is <0 error, 0= finished */
+  int factorSparseLarge (  );
+  /** Does dense phase of factorization
+      return code is <0 error, 0= finished */
   int factorDense (  );
 
   /// Pivots when just one other row so faster?
@@ -508,6 +524,9 @@ protected:
   bool getColumnSpace ( int iColumn,
 			int extraNeeded );
 
+  /** Reorders U so contiguous and in order (if there is space)
+      Returns true if it could */
+  bool reorderU();
   /**  getColumnSpaceIterateR.  Gets space for one extra R element in Column
        may have to do compression  (returns true)
        also moves existing vector */
@@ -632,6 +651,10 @@ protected:
   /** Updates part of column transpose (BTRANU) when sparse,
       assumes index is sorted i.e. region is correct */
   void updateColumnTransposeUSparse ( CoinIndexedVector * region) const;
+  /** Updates part of column transpose (BTRANU) by column
+      assumes index is sorted i.e. region is correct */
+  void updateColumnTransposeUByColumn ( CoinIndexedVector * region,
+					int smallestIndex) const;
 
   /// Updates part of column transpose (BTRANR)
   void updateColumnTransposeR ( CoinIndexedVector * region ) const;
@@ -676,7 +699,7 @@ protected:
 	  int pivotColumn,
 	  CoinBigIndex pivotRowPosition,
 	  CoinBigIndex pivotColumnPosition,
-	  double work[],
+	  CoinFactorizationDouble work[],
 	  unsigned int workArea2[],
 	  int increment,
 	  int increment2,
@@ -686,11 +709,11 @@ protected:
   int *indexColumnU = indexColumnU_.array();
   CoinBigIndex *startColumnU = startColumnU_.array();
   int *numberInColumn = numberInColumn_.array();
-  double *elementU = elementU_.array();
+  CoinFactorizationDouble *elementU = elementU_.array();
   int *indexRowU = indexRowU_.array();
   CoinBigIndex *startRowU = startRowU_.array();
   int *numberInRow = numberInRow_.array();
-  double *elementL = elementL_.array();
+  CoinFactorizationDouble *elementL = elementL_.array();
   int *indexRowL = indexRowL_.array();
   int *saveColumn = saveColumn_.array();
   int *nextRow = nextRow_.array();
@@ -739,13 +762,13 @@ protected:
 
   if ( l + numberInPivotColumn > lengthAreaL_ ) {
     //need more memory
-    printf("more memory needed in middle of invert\n");
+    if ((messageLevel_&4)!=0) 
+      printf("more memory needed in middle of invert\n");
     return false;
   }
   //l+=currentAreaL_->elementByColumn-elementL;
   CoinBigIndex lSave = l;
 
-  pivotRowL_.array()[numberGoodL_] = pivotRow;
   CoinBigIndex * startColumnL = startColumnL_.array();
   startColumnL[numberGoodL_] = l;	//for luck and first time
   numberGoodL_++;
@@ -757,7 +780,7 @@ protected:
       if ( iRow != pivotRow ) {
 	indexRowL[l] = iRow;
 	elementL[l] = elementU[pivotRowPosition];
-	markRow[iRow] = l - lSave;
+	markRow[iRow] = static_cast<T>(l - lSave);
 	l++;
 	//take out of row list
 	CoinBigIndex start = startRowU[iRow];
@@ -784,7 +807,7 @@ protected:
     for ( i = startColumn; i < pivotRowPosition; i++ ) {
       int iRow = indexRowU[i];
 
-      markRow[iRow] = l - lSave;
+      markRow[iRow] = static_cast<T>(l - lSave);
       indexRowL[l] = iRow;
       elementL[l] = elementU[i];
       l++;
@@ -808,15 +831,15 @@ protected:
   }
   assert (pivotRowPosition<endColumn);
   assert (indexRowU[pivotRowPosition]==pivotRow);
-  double pivotElement = elementU[pivotRowPosition];
-  double pivotMultiplier = 1.0 / pivotElement;
+  CoinFactorizationDouble pivotElement = elementU[pivotRowPosition];
+  CoinFactorizationDouble pivotMultiplier = 1.0 / pivotElement;
 
   pivotRegion_.array()[numberGoodU_] = pivotMultiplier;
   pivotRowPosition++;
   for ( ; pivotRowPosition < endColumn; pivotRowPosition++ ) {
     int iRow = indexRowU[pivotRowPosition];
     
-    markRow[iRow] = l - lSave;
+    markRow[iRow] = static_cast<T>(l - lSave);
     indexRowL[l] = iRow;
     elementL[l] = elementU[pivotRowPosition];
     l++;
@@ -837,12 +860,12 @@ protected:
     numberInRow[iRow]--;
     assert (numberInRow[iRow]>=0);
   }
-  markRow[pivotRow] = largeInteger;
+  markRow[pivotRow] = static_cast<T>(largeInteger);
   //compress pivot column (move pivot to front including saved)
   numberInColumn[pivotColumn] = 0;
   //use end of L for temporary space
   int *indexL = &indexRowL[lSave];
-  double *multipliersL = &elementL[lSave];
+  CoinFactorizationDouble *multipliersL = &elementL[lSave];
 
   //adjust
   int j;
@@ -867,17 +890,17 @@ protected:
     CoinBigIndex startColumn = startColumnU[iColumn];
     CoinBigIndex endColumn = startColumn + numberInColumn[iColumn];
     int iRow = indexRowU[startColumn];
-    double value = elementU[startColumn];
+    CoinFactorizationDouble value = elementU[startColumn];
     double largest;
     CoinBigIndex put = startColumn;
     CoinBigIndex positionLargest = -1;
-    double thisPivotValue = 0.0;
+    CoinFactorizationDouble thisPivotValue = 0.0;
 
     //compress column and find largest not updated
     bool checkLargest;
     int mark = markRow[iRow];
 
-    if ( mark < 0 ) {
+    if ( mark == largeInteger+1 ) {
       largest = fabs ( value );
       positionLargest = put;
       put++;
@@ -904,7 +927,7 @@ protected:
       value = elementU[i];
       int mark = markRow[iRow];
 
-      if ( mark < 0 ) {
+      if ( mark == largeInteger+1 ) {
 	//keep
 	indexRowU[put] = iRow;
 	elementU[put] = value;
@@ -1078,7 +1101,7 @@ protected:
       }
       //put back next one in case zapped
       indexColumnU[startRowU[next]] = saveIndex;
-      markRow[iRow] = -1;
+      markRow[iRow] = static_cast<T>(largeInteger+1);
       number = end - startRowU[iRow];
       numberInRow[iRow] = number;
       deleteLink ( iRow );
@@ -1133,13 +1156,13 @@ protected:
       end += test;
     }
     indexColumnU[startRowU[next]] = saveIndex;
-    markRow[iRow] = -1;
+    markRow[iRow] = static_cast<T>(largeInteger+1);
     number = end - startRowU[iRow];
     numberInRow[iRow] = number;
     deleteLink ( iRow );
     addLink ( iRow, number );
   }
-  markRow[pivotRow] = -1;
+  markRow[pivotRow] = static_cast<T>(largeInteger+1);
   //modify linked list for pivots
   deleteLink ( pivotRow );
   deleteLink ( pivotColumn + numberRows_ );
@@ -1267,7 +1290,7 @@ protected:
   CoinIntArrayWithLength pivotRowL_;
 
   /// Inverses of pivot values
-  CoinDoubleArrayWithLength pivotRegion_;
+  CoinFactorizationDoubleArrayWithLength pivotRegion_;
 
   /// Number of slacks at beginning of U
   int numberSlacks_;
@@ -1288,7 +1311,7 @@ protected:
   CoinBigIndex lengthAreaU_;
 
 /// Elements of U
-  CoinDoubleArrayWithLength elementU_;
+  CoinFactorizationDoubleArrayWithLength elementU_;
 
 /// Row indices of U
   CoinIntArrayWithLength indexRowU_;
@@ -1312,7 +1335,7 @@ protected:
   CoinBigIndex lengthAreaL_;
 
   /// Elements of L
-  CoinDoubleArrayWithLength elementL_;
+  CoinFactorizationDoubleArrayWithLength elementL_;
 
   /// Row indices of L
   CoinIntArrayWithLength indexRowL_;
@@ -1333,7 +1356,7 @@ protected:
   CoinBigIndex lengthAreaR_;
 
   /// Elements of R
-  double *elementR_;
+  CoinFactorizationDouble *elementR_;
 
   /// Row indices for R
   int *indexRowR_;
@@ -1354,7 +1377,7 @@ protected:
   int denseThreshold_;
 
   /// First work area
-  CoinDoubleArrayWithLength workArea_;
+  CoinFactorizationDoubleArrayWithLength workArea_;
 
   /// Second work area
   CoinUnsignedIntArrayWithLength workArea2_;
@@ -1400,7 +1423,7 @@ protected:
   CoinIntArrayWithLength indexColumnL_;
 
   /// Elements in L (row copy)
-  CoinDoubleArrayWithLength elementByRowL_;
+  CoinFactorizationDoubleArrayWithLength elementByRowL_;
 
   /// Sparse regions
   mutable CoinIntArrayWithLength sparse_;
@@ -1427,3 +1450,463 @@ typedef const int cipfint;
 #endif
 #endif
 #endif
+// Extra for ugly include
+#ifdef UGLY_COIN_FACTOR_CODING
+#define FAC_UNSET (FAC_SET+1)
+{
+  goodPivot=false;
+  //store pivot columns (so can easily compress)
+  CoinBigIndex startColumnThis = startColumn[iPivotColumn];
+  CoinBigIndex endColumn = startColumnThis + numberDoColumn + 1;
+  int put = 0;
+  CoinBigIndex startRowThis = startRow[iPivotRow];
+  CoinBigIndex endRow = startRowThis + numberDoRow + 1;
+  if ( pivotColumnPosition < 0 ) {
+    for ( pivotColumnPosition = startRowThis; pivotColumnPosition < endRow; pivotColumnPosition++ ) {
+      int iColumn = indexColumn[pivotColumnPosition];
+      if ( iColumn != iPivotColumn ) {
+	saveColumn[put++] = iColumn;
+      } else {
+        break;
+      }
+    }
+  } else {
+    for (CoinBigIndex i = startRowThis ; i < pivotColumnPosition ; i++ ) {
+      saveColumn[put++] = indexColumn[i];
+    }
+  }
+  assert (pivotColumnPosition<endRow);
+  assert (indexColumn[pivotColumnPosition]==iPivotColumn);
+  pivotColumnPosition++;
+  for ( ; pivotColumnPosition < endRow; pivotColumnPosition++ ) {
+    saveColumn[put++] = indexColumn[pivotColumnPosition];
+  }
+  //take out this bit of indexColumn
+  int next = nextRow[iPivotRow];
+  int last = lastRow[iPivotRow];
+  
+  nextRow[last] = next;
+  lastRow[next] = last;
+  nextRow[iPivotRow] = numberGoodU_;	//use for permute
+  lastRow[iPivotRow] = -2;
+  numberInRow[iPivotRow] = 0;
+  //store column in L, compress in U and take column out
+  CoinBigIndex l = lengthL_;
+  // **** HORRID coding coming up but a goto seems best!
+  {
+    if ( l + numberDoColumn > lengthAreaL_ ) {
+      //need more memory
+      if ((messageLevel_&4)!=0) 
+	printf("more memory needed in middle of invert\n");
+      goto BAD_PIVOT;
+    }
+    //l+=currentAreaL_->elementByColumn-elementL;
+    CoinBigIndex lSave = l;
+    
+    CoinBigIndex * startColumnL = startColumnL_.array();
+    startColumnL[numberGoodL_] = l;	//for luck and first time
+    numberGoodL_++;
+    startColumnL[numberGoodL_] = l + numberDoColumn;
+    lengthL_ += numberDoColumn;
+    if ( pivotRowPosition < 0 ) {
+      for ( pivotRowPosition = startColumnThis; pivotRowPosition < endColumn; pivotRowPosition++ ) {
+	int iRow = indexRow[pivotRowPosition];
+	if ( iRow != iPivotRow ) {
+	  indexRowL[l] = iRow;
+	  elementL[l] = element[pivotRowPosition];
+	  markRow[iRow] = l - lSave;
+	  l++;
+	  //take out of row list
+	  CoinBigIndex start = startRow[iRow];
+	  CoinBigIndex end = start + numberInRow[iRow];
+	  CoinBigIndex where = start;
+	  
+	  while ( indexColumn[where] != iPivotColumn ) {
+	    where++;
+	  }			/* endwhile */
+#if DEBUG_COIN
+	  if ( where >= end ) {
+	    abort (  );
+	  }
+#endif
+	  indexColumn[where] = indexColumn[end - 1];
+	  numberInRow[iRow]--;
+	} else {
+	  break;
+	}
+      }
+    } else {
+      CoinBigIndex i;
+      
+      for ( i = startColumnThis; i < pivotRowPosition; i++ ) {
+	int iRow = indexRow[i];
+	
+	markRow[iRow] = l - lSave;
+	indexRowL[l] = iRow;
+	elementL[l] = element[i];
+	l++;
+	//take out of row list
+	CoinBigIndex start = startRow[iRow];
+	CoinBigIndex end = start + numberInRow[iRow];
+	CoinBigIndex where = start;
+	
+	while ( indexColumn[where] != iPivotColumn ) {
+	  where++;
+	}				/* endwhile */
+#if DEBUG_COIN
+	if ( where >= end ) {
+	  abort (  );
+	}
+#endif
+	indexColumn[where] = indexColumn[end - 1];
+	numberInRow[iRow]--;
+	assert (numberInRow[iRow]>=0);
+      }
+    }
+    assert (pivotRowPosition<endColumn);
+    assert (indexRow[pivotRowPosition]==iPivotRow);
+    CoinFactorizationDouble pivotElement = element[pivotRowPosition];
+    CoinFactorizationDouble pivotMultiplier = 1.0 / pivotElement;
+    
+    pivotRegion_.array()[numberGoodU_] = pivotMultiplier;
+    pivotRowPosition++;
+    for ( ; pivotRowPosition < endColumn; pivotRowPosition++ ) {
+      int iRow = indexRow[pivotRowPosition];
+      
+      markRow[iRow] = l - lSave;
+      indexRowL[l] = iRow;
+      elementL[l] = element[pivotRowPosition];
+      l++;
+      //take out of row list
+      CoinBigIndex start = startRow[iRow];
+      CoinBigIndex end = start + numberInRow[iRow];
+      CoinBigIndex where = start;
+      
+      while ( indexColumn[where] != iPivotColumn ) {
+	where++;
+      }				/* endwhile */
+#if DEBUG_COIN
+      if ( where >= end ) {
+	abort (  );
+      }
+#endif
+      indexColumn[where] = indexColumn[end - 1];
+      numberInRow[iRow]--;
+      assert (numberInRow[iRow]>=0);
+    }
+    markRow[iPivotRow] = FAC_SET;
+    //compress pivot column (move pivot to front including saved)
+    numberInColumn[iPivotColumn] = 0;
+    //use end of L for temporary space
+    int *indexL = &indexRowL[lSave];
+    CoinFactorizationDouble *multipliersL = &elementL[lSave];
+    
+    //adjust
+    int j;
+    
+    for ( j = 0; j < numberDoColumn; j++ ) {
+      multipliersL[j] *= pivotMultiplier;
+    }
+    //zero out fill
+    CoinBigIndex iErase;
+    for ( iErase = 0; iErase < increment2 * numberDoRow;
+	  iErase++ ) {
+      workArea2[iErase] = 0;
+    }
+    CoinBigIndex added = numberDoRow * numberDoColumn;
+    unsigned int *temp2 = workArea2;
+    int * nextColumn = nextColumn_.array();
+    
+    //pack down and move to work
+    int jColumn;
+    for ( jColumn = 0; jColumn < numberDoRow; jColumn++ ) {
+      int iColumn = saveColumn[jColumn];
+      CoinBigIndex startColumnThis = startColumn[iColumn];
+      CoinBigIndex endColumn = startColumnThis + numberInColumn[iColumn];
+      int iRow = indexRow[startColumnThis];
+      CoinFactorizationDouble value = element[startColumnThis];
+      double largest;
+      CoinBigIndex put = startColumnThis;
+      CoinBigIndex positionLargest = -1;
+      CoinFactorizationDouble thisPivotValue = 0.0;
+      
+      //compress column and find largest not updated
+      bool checkLargest;
+      int mark = markRow[iRow];
+      
+      if ( mark == FAC_UNSET ) {
+	largest = fabs ( value );
+	positionLargest = put;
+	put++;
+	checkLargest = false;
+      } else {
+	//need to find largest
+	largest = 0.0;
+	checkLargest = true;
+	if ( mark != FAC_SET ) {
+	  //will be updated
+	  workArea[mark] = value;
+	  int word = mark >> COINFACTORIZATION_SHIFT_PER_INT;
+	  int bit = mark & COINFACTORIZATION_MASK_PER_INT;
+	  
+	  temp2[word] = temp2[word] | ( 1 << bit );	//say already in counts
+	  added--;
+	} else {
+	  thisPivotValue = value;
+	}
+      }
+      CoinBigIndex i;
+      for ( i = startColumnThis + 1; i < endColumn; i++ ) {
+	iRow = indexRow[i];
+	value = element[i];
+	int mark = markRow[iRow];
+	
+	if ( mark == FAC_UNSET ) {
+	  //keep
+	  indexRow[put] = iRow;
+	  element[put] = value;
+	  if ( checkLargest ) {
+	    double absValue = fabs ( value );
+	    
+	    if ( absValue > largest ) {
+	      largest = absValue;
+	      positionLargest = put;
+	    }
+	  }
+	  put++;
+	} else if ( mark != FAC_SET ) {
+	  //will be updated
+	  workArea[mark] = value;
+	  int word = mark >> COINFACTORIZATION_SHIFT_PER_INT;
+	  int bit = mark & COINFACTORIZATION_MASK_PER_INT;
+	  
+	  temp2[word] = temp2[word] | ( 1 << bit );	//say already in counts
+	  added--;
+	} else {
+	  thisPivotValue = value;
+	}
+      }
+      //slot in pivot
+      element[put] = element[startColumnThis];
+      indexRow[put] = indexRow[startColumnThis];
+      if ( positionLargest == startColumnThis ) {
+	positionLargest = put;	//follow if was largest
+      }
+      put++;
+      element[startColumnThis] = thisPivotValue;
+      indexRow[startColumnThis] = iPivotRow;
+      //clean up counts
+      startColumnThis++;
+      numberInColumn[iColumn] = put - startColumnThis;
+      int * numberInColumnPlus = numberInColumnPlus_.array();
+      numberInColumnPlus[iColumn]++;
+      startColumn[iColumn]++;
+      //how much space have we got
+      int next = nextColumn[iColumn];
+      CoinBigIndex space;
+      
+      space = startColumn[next] - put - numberInColumnPlus[next];
+      //assume no zero elements
+      if ( numberDoColumn > space ) {
+	//getColumnSpace also moves fixed part
+	if ( !getColumnSpace ( iColumn, numberDoColumn ) ) {
+	  goto BAD_PIVOT;
+	}
+	//redo starts
+	positionLargest = positionLargest + startColumn[iColumn] - startColumnThis;
+	startColumnThis = startColumn[iColumn];
+	put = startColumnThis + numberInColumn[iColumn];
+      }
+      double tolerance = zeroTolerance_;
+      
+      int *nextCount = nextCount_.array();
+      for ( j = 0; j < numberDoColumn; j++ ) {
+	value = workArea[j] - thisPivotValue * multipliersL[j];
+	double absValue = fabs ( value );
+	
+	if ( absValue > tolerance ) {
+	  workArea[j] = 0.0;
+	  element[put] = value;
+	  indexRow[put] = indexL[j];
+	  if ( absValue > largest ) {
+	    largest = absValue;
+	    positionLargest = put;
+	  }
+	  put++;
+	} else {
+	  workArea[j] = 0.0;
+	  added--;
+	  int word = j >> COINFACTORIZATION_SHIFT_PER_INT;
+	  int bit = j & COINFACTORIZATION_MASK_PER_INT;
+	  
+	  if ( temp2[word] & ( 1 << bit ) ) {
+	    //take out of row list
+	    iRow = indexL[j];
+	    CoinBigIndex start = startRow[iRow];
+	    CoinBigIndex end = start + numberInRow[iRow];
+	    CoinBigIndex where = start;
+	    
+	    while ( indexColumn[where] != iColumn ) {
+	      where++;
+	    }			/* endwhile */
+#if DEBUG_COIN
+	    if ( where >= end ) {
+	      abort (  );
+	    }
+#endif
+	    indexColumn[where] = indexColumn[end - 1];
+	    numberInRow[iRow]--;
+	  } else {
+	    //make sure won't be added
+	    int word = j >> COINFACTORIZATION_SHIFT_PER_INT;
+	    int bit = j & COINFACTORIZATION_MASK_PER_INT;
+	    
+	    temp2[word] = temp2[word] | ( 1 << bit );	//say already in counts
+	  }
+	}
+      }
+      numberInColumn[iColumn] = put - startColumnThis;
+      //move largest
+      if ( positionLargest >= 0 ) {
+	value = element[positionLargest];
+	iRow = indexRow[positionLargest];
+	element[positionLargest] = element[startColumnThis];
+	indexRow[positionLargest] = indexRow[startColumnThis];
+	element[startColumnThis] = value;
+	indexRow[startColumnThis] = iRow;
+      }
+      //linked list for column
+      if ( nextCount[iColumn + numberRows_] != -2 ) {
+	//modify linked list
+	deleteLink ( iColumn + numberRows_ );
+	addLink ( iColumn + numberRows_, numberInColumn[iColumn] );
+      }
+      temp2 += increment2;
+    }
+    //get space for row list
+    unsigned int *putBase = workArea2;
+    int bigLoops = numberDoColumn >> COINFACTORIZATION_SHIFT_PER_INT;
+    int i = 0;
+    
+    // do linked lists and update counts
+    while ( bigLoops ) {
+      bigLoops--;
+      int bit;
+      for ( bit = 0; bit < COINFACTORIZATION_BITS_PER_INT; i++, bit++ ) {
+	unsigned int *putThis = putBase;
+	int iRow = indexL[i];
+	
+	//get space
+	int number = 0;
+	int jColumn;
+	
+	for ( jColumn = 0; jColumn < numberDoRow; jColumn++ ) {
+	  unsigned int test = *putThis;
+	  
+	  putThis += increment2;
+	  test = 1 - ( ( test >> bit ) & 1 );
+	  number += test;
+	}
+	int next = nextRow[iRow];
+	CoinBigIndex space;
+	
+	space = startRow[next] - startRow[iRow];
+	number += numberInRow[iRow];
+	if ( space < number ) {
+	  if ( !getRowSpace ( iRow, number ) ) {
+	    goto BAD_PIVOT;
+	  }
+	}
+	// now do
+	putThis = putBase;
+	next = nextRow[iRow];
+	number = numberInRow[iRow];
+	CoinBigIndex end = startRow[iRow] + number;
+	int saveIndex = indexColumn[startRow[next]];
+	
+	//add in
+	for ( jColumn = 0; jColumn < numberDoRow; jColumn++ ) {
+	  unsigned int test = *putThis;
+	  
+	  putThis += increment2;
+	  test = 1 - ( ( test >> bit ) & 1 );
+	  indexColumn[end] = saveColumn[jColumn];
+	  end += test;
+	}
+	//put back next one in case zapped
+	indexColumn[startRow[next]] = saveIndex;
+	markRow[iRow] = FAC_UNSET;
+	number = end - startRow[iRow];
+	numberInRow[iRow] = number;
+	deleteLink ( iRow );
+	addLink ( iRow, number );
+      }
+      putBase++;
+    }				/* endwhile */
+    int bit;
+    
+    for ( bit = 0; i < numberDoColumn; i++, bit++ ) {
+      unsigned int *putThis = putBase;
+      int iRow = indexL[i];
+      
+      //get space
+      int number = 0;
+      int jColumn;
+      
+      for ( jColumn = 0; jColumn < numberDoRow; jColumn++ ) {
+	unsigned int test = *putThis;
+	
+	putThis += increment2;
+	test = 1 - ( ( test >> bit ) & 1 );
+	number += test;
+      }
+      int next = nextRow[iRow];
+      CoinBigIndex space;
+      
+      space = startRow[next] - startRow[iRow];
+      number += numberInRow[iRow];
+      if ( space < number ) {
+	if ( !getRowSpace ( iRow, number ) ) {
+	  goto BAD_PIVOT;
+	}
+      }
+      // now do
+      putThis = putBase;
+      next = nextRow[iRow];
+      number = numberInRow[iRow];
+      CoinBigIndex end = startRow[iRow] + number;
+      int saveIndex;
+      
+      saveIndex = indexColumn[startRow[next]];
+      
+      //add in
+      for ( jColumn = 0; jColumn < numberDoRow; jColumn++ ) {
+	unsigned int test = *putThis;
+	
+	putThis += increment2;
+	test = 1 - ( ( test >> bit ) & 1 );
+	
+	indexColumn[end] = saveColumn[jColumn];
+	end += test;
+      }
+      indexColumn[startRow[next]] = saveIndex;
+      markRow[iRow] = FAC_UNSET;
+      number = end - startRow[iRow];
+      numberInRow[iRow] = number;
+      deleteLink ( iRow );
+      addLink ( iRow, number );
+    }
+    markRow[iPivotRow] = FAC_UNSET;
+    //modify linked list for pivots
+    deleteLink ( iPivotRow );
+    deleteLink ( iPivotColumn + numberRows_ );
+    totalElements_ += added;
+    goodPivot= true;
+    // **** UGLY UGLY UGLY
+  }
+ BAD_PIVOT:
+
+  ;
+}
+#undef FAC_UNSET
+#endif
