@@ -62,6 +62,18 @@ typedef long CoinBigIndex;
 typedef long long CoinBigIndex;
 #endif
 
+//=============================================================================
+#ifndef COIN_BIG_DOUBLE 
+#define COIN_BIG_DOUBLE 0
+#endif
+#if COIN_BIG_DOUBLE==0
+typedef double CoinFactorizationDouble;
+#elif COIN_BIG_DOUBLE==1
+typedef long double CoinFactorizationDouble;
+#else
+typedef double CoinFactorizationDouble;
+#endif
+
 //=============================================================================
 // Plus infinity (double and int)
 #ifndef COIN_DBL_MAX
@@ -69,26 +81,11 @@ typedef long long CoinBigIndex;
 #endif
 
 #ifndef COIN_INT_MAX
-#define COIN_INT_MAX ((int)((~((unsigned int)0)) >> 1))
+#define COIN_INT_MAX (static_cast<int>((~(static_cast<unsigned int>(0))) >> 1))
 #endif
 
-//=============================================================================
-
-#if defined(_MSC_VER)
-//  # define CoinIsnan   _isnan
-//  # define CoinFinite  _finite
-// Turn off compiler warning about long names
-#  pragma warning(disable:4786)
-#if !defined(min)
-#define min(a,b)  (((a) < (b)) ? (a) : (b))
-#endif
-#if !defined(max)
-#define max(a,b)  (((a) > (b)) ? (a) : (b))
-#endif
-#else
-// Put standard min and max here
-using std::min;
-using std::max;
+#ifndef COIN_INT_MAX_AS_DOUBLE
+#define COIN_INT_MAX_AS_DOUBLE (static_cast<double>((~(static_cast<unsigned int>(0))) >> 1))
 #endif
 
 //=============================================================================
