@@ -1,4 +1,4 @@
-// Last edit: 2/10/07
+// Last edit: 11/5/08
 //
 // Name:     CoinLpIO.cpp; Support for Lp files
 // Author:   Francois Margot
@@ -12,7 +12,10 @@
 #include <cfloat>
 #include <cctype>
 #include <cassert>
+#include <string>
+#include <cstdarg>
 
+#include "CoinError.hpp"
 #include "CoinHelperFunctions.hpp"
 #include "CoinPackedMatrix.hpp"
 #include "CoinLpIO.hpp"
@@ -211,7 +214,7 @@ CoinLpIO::convertBoundToSense(const double lower, const double upper,
 {
   if(rowsense_  == NULL) {
     int nr=numberRows_;
-    rowsense_ = (char *) malloc(nr*sizeof(char));
+    rowsense_ = reinterpret_cast<char *> (malloc(nr*sizeof(char)));
     
     double dum1,dum2;
     int i;
@@ -227,7 +230,7 @@ const double * CoinLpIO::getRightHandSide() const
 {
   if(rhs_==NULL) {
     int nr=numberRows_;
-    rhs_ = (double *) malloc(nr*sizeof(double));
+    rhs_ = reinterpret_cast<double *> (malloc(nr*sizeof(double)));
 
     char dum1;
     double dum2;
@@ -244,7 +247,7 @@ const double * CoinLpIO::getRowRange() const
 {
   if (rowrange_ == NULL) {
     int nr=numberRows_;
-    rowrange_ = (double *) malloc(nr*sizeof(double));
+    rowrange_ = reinterpret_cast<double *> (malloc(nr*sizeof(double)));
     std::fill(rowrange_,rowrange_+nr,0.0);
 
     char dum1;
@@ -402,12 +405,13 @@ double CoinLpIO::getInfinity() const
 /************************************************************************/
 void CoinLpIO::setInfinity(const double value) 
 {
-  if (value >= 1.020) {
+  if (value >= 1.0e20) {
     infinity_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setInfinity(): value: %f\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %f\n", value);
+    throw CoinError(str, "setInfinity", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -424,8 +428,9 @@ void CoinLpIO::setEpsilon(const double value)
     epsilon_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setEpsilon(): value: %f\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %f\n", value);
+    throw CoinError(str, "setEpsilon", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -442,8 +447,9 @@ void CoinLpIO::setNumberAcross(const int value)
     numberAcross_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setNumberAcross(): value: %d\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %d\n", value);
+    throw CoinError(str, "setNumberAcross", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -460,8 +466,9 @@ void CoinLpIO::setDecimals(const int value)
     decimals_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setDecimals(): value: %d\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %d\n", value);
+    throw CoinError(str, "setDecimals", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -509,11 +516,11 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
   numberColumns_ = matrixByRow_->getNumCols();
   numberRows_ = matrixByRow_->getNumRows();
   
-  rowlower_ = (double *) malloc (numberRows_ * sizeof(double));
-  rowupper_ = (double *) malloc (numberRows_ * sizeof(double));
-  collower_ = (double *) malloc (numberColumns_ * sizeof(double));
-  colupper_ = (double *) malloc (numberColumns_ * sizeof(double));
-  objective_ = (double *) malloc (numberColumns_ * sizeof(double));
+  rowlower_ = reinterpret_cast<double *> (malloc (numberRows_ * sizeof(double)));
+  rowupper_ = reinterpret_cast<double *> (malloc (numberRows_ * sizeof(double)));
+  collower_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
+  colupper_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
+  objective_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
   std::copy(rowlb, rowlb + numberRows_, rowlower_);
   std::copy(rowub, rowub + numberRows_, rowupper_);
   std::copy(collb, collb + numberColumns_, collower_);
@@ -521,7 +528,7 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
   std::copy(obj_coeff, obj_coeff + numberColumns_, objective_);
 
   if (is_integer) {
-    integerType_ = (char *) malloc (numberColumns_ * sizeof(char));
+    integerType_ = reinterpret_cast<char *> (malloc (numberColumns_ * sizeof(char)));
     std::copy(is_integer, is_integer + numberColumns_, integerType_);
   } 
   else {
@@ -540,7 +547,7 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
 void CoinLpIO::setDefaultRowNames() {
 
   int i, nrow = getNumRows();
-  char **defaultRowNames = (char **) malloc ((nrow+1) * sizeof(char *));
+  char **defaultRowNames = reinterpret_cast<char **> (malloc ((nrow+1) * sizeof(char *)));
   char buff[1024];
 
   for(i=0; i<nrow; i++) {
@@ -565,7 +572,7 @@ void CoinLpIO::setDefaultRowNames() {
 void CoinLpIO::setDefaultColNames() {
 
   int j, ncol = getNumCols();
-  char **defaultColNames = (char **) malloc (ncol * sizeof(char *));
+  char **defaultColNames = reinterpret_cast<char **> (malloc (ncol * sizeof(char *)));
   char buff[256];
 
   for(j=0; j<ncol; j++) {
@@ -660,16 +667,16 @@ CoinLpIO::writeLp(const char *filename, const double epsilon,
 		  const int numberAcross, const int decimals,
 		  const bool useRowNames) {
 
-   FILE *fp = NULL;
-   fp = fopen(filename,"w");
-   if (!fp) {
-     printf("### ERROR: in CoinLpIO::writeLP(): unable to open file %s\n",
-	    filename);
-     exit(1);
-   }
-   int nerr = writeLp(fp, epsilon, numberAcross, decimals, useRowNames);
-   fclose(fp);
-   return(nerr);
+  FILE *fp = NULL;
+  fp = fopen(filename,"w");
+  if (!fp) {
+    char str[8192];
+    sprintf(str,"### ERROR: unable to open file %s\n", filename);
+    throw CoinError(str, "writeLP", "CoinLpIO", __FILE__, __LINE__);
+  }
+  int nerr = writeLp(fp, epsilon, numberAcross, decimals, useRowNames);
+  fclose(fp);
+  return(nerr);
 }
 
 /************************************************************************/
@@ -688,16 +695,16 @@ CoinLpIO::writeLp(FILE *fp, const double epsilon,
 int
 CoinLpIO::writeLp(const char *filename, const bool useRowNames)
 {
-   FILE *fp = NULL;
-   fp = fopen(filename,"w");
-   if (!fp) {
-     printf("### ERROR: in CoinLpIO::writeLP(): unable to open file %s\n",
-	    filename);
-     exit(1);
-   }
-   int nerr = writeLp(fp, useRowNames);
-   fclose(fp);
-   return(nerr);
+  FILE *fp = NULL;
+  fp = fopen(filename,"w");
+  if (!fp) {
+    char str[8192];
+    sprintf(str,"### ERROR: unable to open file %s\n", filename);
+    throw CoinError(str, "writeLP", "CoinLpIO", __FILE__, __LINE__);
+  }
+  int nerr = writeLp(fp, useRowNames);
+  fclose(fp);
+  return(nerr);
 }
 
 /************************************************************************/
@@ -728,7 +735,7 @@ CoinLpIO::writeLp(FILE *fp, const bool useRowNames)
 
    if(rowNames == NULL) {
      loc_row_names = 1;
-     prowNames = (char **) malloc ((nrow+1) * sizeof(char *));
+     prowNames = reinterpret_cast<char **> (malloc ((nrow+1) * sizeof(char *)));
 
      for (j=0; j<nrow; j++) {
        sprintf(buff, "cons%d", j);
@@ -740,7 +747,7 @@ CoinLpIO::writeLp(FILE *fp, const bool useRowNames)
 
    if(colNames == NULL) {
      loc_col_names = 1;
-     pcolNames = (char **) malloc (ncol * sizeof(char *));
+     pcolNames = reinterpret_cast<char **> (malloc (ncol * sizeof(char *)));
 
      for (j=0; j<ncol; j++) {
        sprintf(buff, "x%d", j);
@@ -958,10 +965,11 @@ CoinLpIO::find_obj(FILE *fp) const {
 
     scan_next(buff, fp);
     lbuff = strlen(buff);
-
+    
     if(feof(fp)) {
-      printf("### ERROR: CoinLpIO: find_obj(): Unable to locate objective function\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Unable to locate objective function\n");
+      throw CoinError(str, "find_obj", "CoinLpIO", __FILE__, __LINE__);
     }
   }
 
@@ -1039,6 +1047,18 @@ CoinLpIO::is_free(const char *buff) const {
   return(0);
 } /* is_free */
 
+/*************************************************************************/
+int 
+CoinLpIO::is_inf(const char *buff) const {
+
+  unsigned lbuff = strlen(buff);
+
+  if((lbuff == 3) && (CoinStrNCaseCmp(buff, "inf", 3) == 0)) {
+    return(1);
+  }
+  return(0);
+} /* is_inf */
+
 /*************************************************************************/
 int 
 CoinLpIO::is_comment(const char *buff) const {
@@ -1055,12 +1075,14 @@ CoinLpIO::skip_comment(char *buff, FILE *fp) const {
 
   while(strcspn(buff, "\n") == strlen(buff)) { // end of line not read yet
     if(feof(fp)) {
-      printf("### ERROR: CoinLpIO::skip_comment(): end of file reached while skipping comment\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: end of file reached while skipping comment\n");
+      throw CoinError(str, "skip_comment", "CoinLpIO", __FILE__, __LINE__);
     }
     if(ferror(fp)) {
-      printf("### ERROR: CoinLpIO::skip_comment(): error while skipping comment\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: error while skipping comment\n");
+      throw CoinError(str, "skip_comment", "CoinLpIO", __FILE__, __LINE__);
     }
     fgets(buff, sizeof(buff), fp);    
   } 
@@ -1119,7 +1141,7 @@ CoinLpIO::is_invalid_name(const char *name,
     return(3);
   }
 
-  if((is_keyword(name)) || (is_free(name))) {
+  if((is_keyword(name)) || (is_free(name) || (is_inf(name)))) {
     return(4);
   }
 
@@ -1137,8 +1159,10 @@ CoinLpIO::are_invalid_names(char const * const * const vnames,
   const char * rSense = getRowSense();
 
   if((check_ranged) && (card_vnames != nrows+1)) {
-    printf("### ERROR: CoinLpIO::are_invalid_names(): card_vnames: %d   number of rows: %d\n", card_vnames, getNumRows());
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: card_vnames: %d   number of rows: %d\n",
+	    card_vnames, getNumRows());
+    throw CoinError(str, "are_invalid_names", "CoinLpIO", __FILE__, __LINE__);
   }
 
   for(i=0; i<card_vnames; i++) {
@@ -1151,7 +1175,7 @@ CoinLpIO::are_invalid_names(char const * const * const vnames,
     }
     flag = is_invalid_name(vnames[i], is_ranged);
     if(flag) {
-      printf("### WARNING: CoinLpIO::are_invalid_names(): invalid name: vnames[%d]: %s\n",
+      printf("### WARNING: CoinLpIO::are_invalid_names(): Invalid name: vnames[%d]: %s\n",
 	     i, vnames[i]);
       invalid = flag;
     }
@@ -1171,8 +1195,9 @@ CoinLpIO::read_monom_obj(FILE *fp, double *coeff, char **name, int *cnt,
   scan_next(buff, fp);
 
   if(feof(fp)) {
-    printf("### ERROR: CoinLpIO:  read_monom_obj(): Unable to read objective function\n");
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: Unable to read objective function\n");
+    throw CoinError(str, "read_monom_obj", "CoinLpIO", __FILE__, __LINE__);
   }
 
   if(buff[strlen(buff)-1] == ':') {
@@ -1325,8 +1350,8 @@ CoinLpIO::realloc_coeff(double **coeff, char ***colNames,
   
   *maxcoeff *= 5;
 
-  *colNames = (char **) realloc ((*colNames), (*maxcoeff+1) * sizeof(char *));
-  *coeff = (double *) realloc ((*coeff), (*maxcoeff+1) * sizeof(double));
+  *colNames = reinterpret_cast<char **> (realloc ((*colNames), (*maxcoeff+1) * sizeof(char *)));
+  *coeff = reinterpret_cast<double *> (realloc ((*coeff), (*maxcoeff+1) * sizeof(double)));
 
 } /* realloc_coeff */
 
@@ -1336,11 +1361,11 @@ CoinLpIO::realloc_row(char ***rowNames, int **start, double **rhs,
 		      double **rowlow, double **rowup, int *maxrow) const {
 
   *maxrow *= 5;
-  *rowNames = (char **) realloc ((*rowNames), (*maxrow+1) * sizeof(char *));
-  *start = (int *) realloc ((*start), (*maxrow+1) * sizeof(int));
-  *rhs = (double *) realloc ((*rhs), (*maxrow+1) * sizeof(double));
-  *rowlow = (double *) realloc ((*rowlow), (*maxrow+1) * sizeof(double));
-  *rowup = (double *) realloc ((*rowup), (*maxrow+1) * sizeof(double));
+  *rowNames = reinterpret_cast<char **> (realloc ((*rowNames), (*maxrow+1) * sizeof(char *)));
+  *start = reinterpret_cast<int *> (realloc ((*start), (*maxrow+1) * sizeof(int)));
+  *rhs = reinterpret_cast<double *> (realloc ((*rhs), (*maxrow+1) * sizeof(double)));
+  *rowlow = reinterpret_cast<double *> (realloc ((*rowlow), (*maxrow+1) * sizeof(double)));
+  *rowup = reinterpret_cast<double *> (realloc ((*rowup), (*maxrow+1) * sizeof(double)));
 
 } /* realloc_row */
 
@@ -1350,9 +1375,9 @@ CoinLpIO::realloc_col(double **collow, double **colup, char **is_int,
 		      int *maxcol) const {
   
   *maxcol += 100;
-  *collow = (double *) realloc ((*collow), (*maxcol+1) * sizeof(double));
-  *colup = (double *) realloc ((*colup), (*maxcol+1) * sizeof(double));
-  *is_int = (char *) realloc ((*is_int), (*maxcol+1) * sizeof(char));
+  *collow = reinterpret_cast<double *> (realloc ((*collow), (*maxcol+1) * sizeof(double)));
+  *colup = reinterpret_cast<double *> (realloc ((*colup), (*maxcol+1) * sizeof(double)));
+  *is_int = reinterpret_cast<char *> (realloc ((*is_int), (*maxcol+1) * sizeof(char)));
 
 } /* realloc_col */
 
@@ -1383,8 +1408,9 @@ CoinLpIO::read_row(FILE *fp, char *buff,
     scan_next(start_str, fp);
 
     if(feof(fp)) {
-      printf("### ERROR: CoinLpIO:  read_monom_row(): Unable to read row monomial\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Unable to read row monomial\n");
+      throw CoinError(str, "read_monom_row", "CoinLpIO", __FILE__, __LINE__);
     }
   }
   (*cnt_coeff)--;
@@ -1452,9 +1478,9 @@ CoinLpIO::readLp(const char *filename)
 {
   FILE *fp = fopen(filename, "r");
   if(!fp) {
-    printf("### ERROR: CoinLpIO:  Unable to open file %s for reading\n",
-	   filename);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: Unable to open file %s for reading\n", filename);
+    throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
   }
   readLp(fp);
   fclose(fp);
@@ -1483,14 +1509,14 @@ CoinLpIO::readLp(FILE* fp)
   int objsense, cnt_coeff = 0, cnt_row = 0, cnt_obj = 0;
   char *objName = NULL;
 
-  char **colNames = (char **) malloc ((maxcoeff+1) * sizeof(char *));
-  double *coeff = (double *) malloc ((maxcoeff+1) * sizeof(double));
+  char **colNames = reinterpret_cast<char **> (malloc ((maxcoeff+1) * sizeof(char *)));
+  double *coeff = reinterpret_cast<double *> (malloc ((maxcoeff+1) * sizeof(double)));
 
-  char **rowNames = (char **) malloc ((maxrow+1) * sizeof(char *));
-  int *start = (int *) malloc ((maxrow+1) * sizeof(int));
-  double *rhs = (double *) malloc ((maxrow+1) * sizeof(double));
-  double *rowlow = (double *) malloc ((maxrow+1) * sizeof(double));
-  double *rowup = (double *) malloc ((maxrow+1) * sizeof(double));
+  char **rowNames = reinterpret_cast<char **> (malloc ((maxrow+1) * sizeof(char *)));
+  int *start = reinterpret_cast<int *> (malloc ((maxrow+1) * sizeof(int)));
+  double *rhs = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
+  double *rowlow = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
+  double *rowup = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
 
   int i;
 
@@ -1513,8 +1539,9 @@ CoinLpIO::readLp(FILE* fp)
     unsigned lbuff = strlen(buff);
 
     if((lbuff != 2) || (CoinStrNCaseCmp(buff, "to", 2) != 0)) {
-      printf("### ERROR: CoinLpIO::readLp(): Can not locate keyword 'Subject To'\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Can not locate keyword 'Subject To'\n");
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
     }
   }
   
@@ -1559,9 +1586,9 @@ CoinLpIO::readLp(FILE* fp)
 
   int maxcol = numberHash_[1] + 100;
 
-  double *collow = (double *) malloc ((maxcol+1) * sizeof(double));
-  double *colup = (double *) malloc ((maxcol+1) * sizeof(double));
-  char *is_int = (char *) malloc ((maxcol+1) * sizeof(char));
+  double *collow = reinterpret_cast<double *> (malloc ((maxcol+1) * sizeof(double)));
+  double *colup = reinterpret_cast<double *> (malloc ((maxcol+1) * sizeof(double)));
+  char *is_int = reinterpret_cast<char *> (malloc ((maxcol+1) * sizeof(char)));
   int has_int = 0;
 
   for (i=0; i<maxcol; i++) {
@@ -1585,8 +1612,8 @@ CoinLpIO::readLp(FILE* fp)
 	int mult = 1;
 	char *start_str = buff;
 
-	if(buff[0] == '-') {
-	  mult = -1;
+	if(buff[0] == '-' || buff[0] == '+') {
+	  mult = (buff[0] == '-') ? -1 : +1;
 	  if(strlen(buff) == 1) {
 	    scan_next(buff, fp);
 	    start_str = buff;
@@ -1596,13 +1623,24 @@ CoinLpIO::readLp(FILE* fp)
 	  }
 	}
 
+	int scan_sense = 0;
 	if(first_is_number(start_str)) {
 	  bnd1 = mult * atof(start_str);
+	  scan_sense = 1;
+	}
+	else {
+	  if(is_inf(start_str)) {
+	    bnd1 = mult * lp_inf;
+	    scan_sense = 1;
+	  }
+	}
+	if(scan_sense) {
 	  scan_next(buff, fp);
 	  read_sense1 = is_sense(buff);
 	  if(read_sense1 < 0) {
-	    printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; expect a sense, get: %s\n", buff);
-	    exit(1);
+	    char str[8192];
+	    sprintf(str,"### ERROR: Bounds; expect a sense, get: %s\n", buff);
+	    throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
 	  }
 	  scan_next(buff, fp);
 	}
@@ -1644,29 +1682,33 @@ CoinLpIO::readLp(FILE* fp)
 	      scan_next(buff, fp);
 	    }
 	    else {
-	      if (strncmp(start_str,"inf",3)&&strncmp(start_str,"Inf",3)) {
-		printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; expect a number, get: %s\n", buff);
-		exit(1);
-	      } else {
-		// infinity
-		bnd2 = DBL_MAX;
+	      if(is_inf(start_str)) {
+		bnd2 = mult * lp_inf;
 		scan_next(buff, fp);
 	      }
+	      else {
+		char str[8192];
+		sprintf(str,"### ERROR: Bounds; expect a number, get: %s\n",
+			buff);
+		throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
+	      }
 	    }
 	  }
 
 	  if((read_sense1 > -1) && (read_sense2 > -1)) {
 	    if(read_sense1 != read_sense2) {
-	      printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; \n\
-variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2);
-	      exit(1);
+	      char str[8192];
+	      sprintf(str,"### ERROR: Bounds; variable: %s read_sense1: %d  read_sense2: %d\n",
+		      buff, read_sense1, read_sense2);
+	      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
 	    }
 	    else {
 	      if(read_sense1 == 1) {
 		if(fabs(bnd1 - bnd2) > lp_eps) {
-		  printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; \nvariable: %s read_sense1: %d  read_sense2: %d  bnd1: %f  bnd2: %f\n", 
-			 buff, read_sense1, read_sense2, bnd1, bnd2);
-		  exit(1);
+		  char str[8192];
+		  sprintf(str,"### ERROR: Bounds; variable: %s read_sense1: %d  read_sense2: %d  bnd1: %f  bnd2: %f\n", 
+			  buff, read_sense1, read_sense2, bnd1, bnd2);
+		  throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
 		}
 		collow[icol] = bnd1;
 		colup[icol] = bnd1;
@@ -1710,7 +1752,7 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 	icol = findHash(buff, 1);
 
 #ifdef LPIO_DEBUG
-	printf("CoinLpIO::readLp(): integer: colname: (%s)  icol: %d\n", 
+	printf("CoinLpIO::readLp(): Integer: colname: (%s)  icol: %d\n", 
 	       buff, icol);
 #endif
 
@@ -1723,7 +1765,7 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 	  }
 	  
 #ifdef LPIO_DEBUG
-	  printf("CoinLpIO::readLp(): integer: colname: (%s)  icol: %d\n", 
+	  printf("CoinLpIO::readLp(): Integer: colname: (%s)  icol: %d\n", 
 		 buff, icol);
 #endif
 	  
@@ -1776,9 +1818,9 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
     case 4: done = 1; break;
       
     default: 
-      printf("### ERROR: CoinLpIO::readLp(): Lost while reading: (%s)\n",
-	     buff);
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Lost while reading: (%s)\n", buff);
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
       break;
     }
   }
@@ -1787,7 +1829,7 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
   printf("CoinLpIO::readLp(): Done with reading the Lp file\n");
 #endif
 
-  int *ind = (int *) malloc ((maxcoeff+1) * sizeof(int));
+  int *ind = reinterpret_cast<int *> (malloc ((maxcoeff+1) * sizeof(int)));
 
   for(i=0; i<cnt_coeff; i++) {
     ind[i] = findHash(colNames[i], 1);
@@ -1798,26 +1840,34 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 #endif
 
     if(ind[i] < 0) {
-      printf("### ERROR: CoinLpIO::readLp(): Problem with hash tables\nCan not find name %s\n", colNames[i]);
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Hash table: %s not found\n", colNames[i]);
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
     }
   }
   
   numberColumns_ = numberHash_[1];
   numberElements_ = cnt_coeff - start[0];
 
-  double *obj = (double *) malloc (numberColumns_ * sizeof(double));
+  double *obj = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
   memset(obj, 0, numberColumns_ * sizeof(double));
 
   for(i=0; i<cnt_obj; i++) {
     icol = findHash(colNames[i], 1);
     if(icol < 0) {
-      printf("### ERROR: CoinLpIO::readLp(): Problem with hash tables\nCan not find name %s (objective function)\n", colNames[i]);
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Hash table: %s (obj) not found\n", colNames[i]);
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
     }
     obj[icol] = objsense * coeff[i];
   }
 
+  if (objsense == -1) {
+    printf("### WARNING: CoinLpIO::readLp(): Maximization problem reformulated as minimization\n");
+    objectiveOffset_ = -objectiveOffset_;
+  }
+
+
   for(i=0; i<cnt_row+1; i++) {
     start[i] -= cnt_obj;
   }
@@ -1983,7 +2033,7 @@ CoinLpIO::startHash(char const * const * const names,
   int maxhash = maxHash_[section];
   COINColumnIndex i, ipos, iput;
 
-  names_[section] = (char **) malloc(maxhash * sizeof(char *));
+  names_[section] = reinterpret_cast<char **> (malloc(maxhash * sizeof(char *)));
   hash_[section] = new CoinHashLink[maxhash];
   
   CoinHashLink * hashThis = hash_[section];
@@ -2043,9 +2093,9 @@ CoinLpIO::startHash(char const * const * const names,
 
 #ifdef LPIO_DEBUG
 	if(j1 > i) {
-	  printf("### ERROR: CoinLpIO::startHash(): j1: %d  i: %d\n",
-		 j1, i);
-	  exit(1);
+	  char str[8192];
+	  sprintf(str,"### ERROR: Hash table: j1: %d  i: %d\n", j1, i);
+	  throw CoinError(str, "startHash", "CoinLpIO", __FILE__, __LINE__);
 	}
 #endif
 
@@ -2067,8 +2117,9 @@ CoinLpIO::startHash(char const * const * const names,
 	    while (1) {
 	      ++iput;
 	      if (iput > maxhash) {
-		printf ( "### ERROR: CoinLpIO::startHash(): too many names\n");
-		exit(1);
+		char str[8192];
+		sprintf(str,"### ERROR: Hash table: too many names\n");
+		throw CoinError(str, "startHash", "CoinLpIO", __FILE__, __LINE__);
 		break;
 	      }
 	      if (hashThis[iput].index == -1) {
@@ -2200,8 +2251,9 @@ CoinLpIO::insertHash(const char *thisName, int section)
 	  while (1) {
 	    ++iput;
 	    if (iput == maxhash) {
-	      printf ( "### ERROR: CoinLpIO::insertHash(): too many names\n");
-	      exit(1);
+	      char str[8192];
+	      sprintf(str,"### ERROR: Hash table: too many names\n");
+	      throw CoinError(str, "insertHash", "CoinLpIO", __FILE__, __LINE__);
 	      break;
 	    }
 	    if (hashThis[iput].index == -1) {
