@@ -41,7 +41,7 @@ slack_doubleton_action::presolve(CoinPresolveMatrix *prob,
   int *hrow		= prob->hrow_;
   CoinBigIndex *mcstrt	= prob->mcstrt_;
   int *hincol		= prob->hincol_;
-  int ncols		= prob->ncols_;
+  //int ncols		= prob->ncols_;
 
   double *clo		= prob->clo_;
   double *cup		= prob->cup_;
@@ -80,7 +80,7 @@ slack_doubleton_action::presolve(CoinPresolveMatrix *prob,
   int nactions = 0;
   notFinished = false;
 
-  int * fixed_cols = new int [ncols];
+  int * fixed_cols = prob->usefulColumnInt_; //new int [ncols];
   int nfixed_cols	= 0;
 
   // this loop can apparently create new singleton rows;
@@ -225,8 +225,10 @@ slack_doubleton_action::presolve(CoinPresolveMatrix *prob,
 	int basisChoice=0;
 	int numberBasic=0;
 	double movement = 0 ;
-	if (prob->columnIsBasic(jcol))
+	if (prob->columnIsBasic(jcol)) {
 	  numberBasic++;
+	  basisChoice=2; // move to row to keep consistent
+	}
 	if (prob->rowIsBasic(irow))
 	  numberBasic++;
 	if (sol[jcol] <= clo[jcol]+ztolzb) {
@@ -240,8 +242,10 @@ slack_doubleton_action::presolve(CoinPresolveMatrix *prob,
 	} else {
 	  basisChoice=1;
 	}
-	if (numberBasic>1||basisChoice)
+	if (numberBasic>1||basisChoice==1)
 	  prob->setColumnStatus(jcol,CoinPrePostsolveMatrix::basic);
+	else if (basisChoice==2)
+	  prob->setRowStatus(irow,CoinPrePostsolveMatrix::basic);
 	if (movement) {
 	  CoinBigIndex k;
 	  for (k = mcstrt[jcol] ; k < mcstrt[jcol]+hincol[jcol] ; k++) {
@@ -280,7 +284,7 @@ slack_doubleton_action::presolve(CoinPresolveMatrix *prob,
 					 true, // arbitrary
 					 next);
   }
-  delete [] fixed_cols;
+  //delete [] fixed_cols;
   if (prob->tuning_) {
     double thisTime=CoinCpuTime();
     int droppedRows = prob->countEmptyRows() - startEmptyRows ;
@@ -507,6 +511,8 @@ slack_singleton_action::presolve(CoinPresolveMatrix *prob,
   double costOffset=0.0;
   for (iLook=0;iLook<numberLook;iLook++) {
     int iCol = look[iLook];
+    if (dcost[iCol])
+      continue;
     if (hincol[iCol] == 1) {
       int iRow=hrow[mcstrt[iCol]];
       double coeff = colels[mcstrt[iCol]];
@@ -646,7 +652,7 @@ slack_singleton_action::presolve(CoinPresolveMatrix *prob,
           if ((sol[iCol] < cup[iCol]-ztolzb&&
 	       sol[iCol] > clo[iCol]+ztolzb)||prob->columnIsBasic(iCol))
             prob->setRowStatus(iRow,CoinPrePostsolveMatrix::basic);
-          prob->setColumnStatusUsingValue(CoinPrePostsolveMatrix::atLowerBound);
+          prob->setColumnStatusUsingValue(iCol);
         }
         // Force column to zero
         clo[iCol]=0.0;
@@ -658,16 +664,18 @@ slack_singleton_action::presolve(CoinPresolveMatrix *prob,
           costOffset += currentLower*rowObjective[iRow];
           prob->dobias_ -= currentLower*rowObjective[iRow];
         }
-        double movement;
-        if (fabs(sol[iCol]-clo[iCol])<fabs(sol[iCol]-cup[iCol])) {
-          movement = clo[iCol]-sol[iCol] ;
-          sol[iCol]=clo[iCol];
-        } else {
-          movement = cup[iCol]-sol[iCol] ;
-          sol[iCol]=cup[iCol];
-        }
-        if (movement) 
-          acts[iRow] += movement*coeff;
+	if (sol) {
+	  double movement;
+	  if (fabs(sol[iCol]-clo[iCol])<fabs(sol[iCol]-cup[iCol])) {
+	    movement = clo[iCol]-sol[iCol] ;
+	    sol[iCol]=clo[iCol];
+	  } else {
+	    movement = cup[iCol]-sol[iCol] ;
+	    sol[iCol]=cup[iCol];
+	  }
+	  if (movement) 
+	    acts[iRow] += movement*coeff;
+	}
         /*
           Remove the row from this col in the col rep.and delink it.
         */
@@ -763,14 +771,36 @@ void slack_singleton_action::postsolve(CoinPostsolveMatrix *prob) const
   CoinBigIndex &free_list		= prob->free_list_;
 
   const double ztolzb	= prob->ztolzb_;
-
+#ifdef CHECK_ONE_ROW
+  {
+    double act=0.0;
+    for (int i=0;i<prob->ncols_;i++) {
+      double solV = sol[i];
+      assert (solV>=clo[i]-ztolzb&&solV<=cup[i]+ztolzb);
+      int j=mcstrt[i];
+      for (int k=0;k<hincol[i];k++) {
+	if (hrow[j]==CHECK_ONE_ROW) {
+	  act += colels[j]*solV;
+	}
+	j=link[j];
+      }
+    }
+    assert (act>=rlo[CHECK_ONE_ROW]-ztolzb&&act<=rup[CHECK_ONE_ROW]+ztolzb);
+    printf("start %g %g %g %g\n",rlo[CHECK_ONE_ROW],act,acts[CHECK_ONE_ROW],rup[CHECK_ONE_ROW]);
+  }
+#endif
   for (const action *f = &actions[nactions-1]; actions<=f; f--) {
     int iRow = f->row;
     double lo0 = f->clo;
     double up0 = f->cup;
     double coeff = f->coeff;
     int iCol = f->col;
-
+    assert (!hincol[iCol]);
+#ifdef CHECK_ONE_ROW
+    if (iRow==CHECK_ONE_ROW)
+      printf("Col %d coeff %g old bounds %g,%g new %g,%g - new rhs %g,%g - act %g\n",
+	     iCol,coeff,clo[iCol],cup[iCol],lo0,up0,f->rlo,f->rup,acts[CHECK_ONE_ROW]);
+#endif
     rlo[iRow] = f->rlo;
     rup[iRow] = f->rup;
 
@@ -794,7 +824,7 @@ void slack_singleton_action::postsolve(CoinPostsolveMatrix *prob) const
       else if (sol[iCol]<clo[iCol]-ztolzb) 
         cMove = clo[iCol]-sol[iCol];
       sol[iCol] += cMove;
-      acts[iRow] -= cMove*coeff;
+      acts[iRow] += cMove*coeff;
       /*
        * Have to compute status.  At most one can be basic. It's possible that
 	 both are nonbasic and nonbasic status must change.
@@ -915,7 +945,26 @@ void slack_singleton_action::postsolve(CoinPostsolveMatrix *prob) const
       mcstrt[iCol] = k;
     }
     hincol[iCol]++;	// right?
-
+#ifdef CHECK_ONE_ROW
+    {
+      double act=0.0;
+      for (int i=0;i<prob->ncols_;i++) {
+	double solV = sol[i];
+	assert (solV>=clo[i]-ztolzb&&solV<=cup[i]+ztolzb);
+	int j=mcstrt[i];
+	for (int k=0;k<hincol[i];k++) {
+	  if (hrow[j]==CHECK_ONE_ROW) {
+	    //printf("c %d el %g sol %g old act %g new %g\n",
+	    //   i,colels[j],solV,act, act+colels[j]*solV);
+	    act += colels[j]*solV;
+	  }
+	  j=link[j];
+	}
+      }
+      assert (act>=rlo[CHECK_ONE_ROW]-ztolzb&&act<=rup[CHECK_ONE_ROW]+ztolzb);
+      printf("rhs now %g %g %g %g\n",rlo[CHECK_ONE_ROW],act,acts[CHECK_ONE_ROW],rup[CHECK_ONE_ROW]);
+    }
+#endif
 
 #   if PRESOLVE_DEBUG
     rdone[iRow] = SLACK_SINGLETON;
